# This is added to faciliting testing without installing secretwallet
# the executable can be started by
# python -m secretwallet <add here the arguments that you need>

from .main import main, configure

main()
#configure()